#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// opisanie ogromnyh chisel

#define HUGE_LEN 10000
#define HUGE_OSN 10

class Big
{
public:
    Big() { zero(); }
    Big(int val) { set(val); }
    Big(const char *val) { set(val); }
    Big(const Big &val) { set(val); }

    Big operator+(const Big &hi) const;
    Big operator-(const Big &hi) const;
    Big operator*(const Big &hi) const;
    int operator>=(const Big &hi) const;

    const char *buff() const { return num; }
    int size() const { return len; }
    void write(FILE *fp = stdout) const;
    int read(FILE *fp);

private:
    void zero();
    void set(int val);
    void set(const char *str);
    void set(const Big &val);

    char num[HUGE_LEN];
    int len;
};

void Big::zero()
{
    memset(num, 0, sizeof(char)*HUGE_LEN);
    len = 1;
}

void Big::set(int val)
{
    zero(); len = 0;
    do {
        num[len++] = val % HUGE_OSN;
        val /= HUGE_OSN;
    } while (val);
}

void Big::set(const char *str)
{
    zero();
    len = strlen(str);
    if (len < 1) return;
    for (int i = len-1; i >= 0; i--)
        num[i] = str[len-1-i] - '0';
}

void Big::set(const Big &val)
{
    memcpy(num, val.num, sizeof(char)*HUGE_LEN);
    len = val.len;
}

Big Big::operator+(const Big &hi) const
{
    Big res;
    int i, max, sum;
    max = (this->len >  hi.len) ? this->len : hi.len;

    for (i = 0; i < max; i++)
    {
        sum = num[i] + hi.num[i] + res.num[i];
        res.num[i] = sum % HUGE_OSN;
        res.num[i+1] = sum / HUGE_OSN;
    }
    res.len = (res.num[max] == 0) ? max : max+1;

    return res;
}

Big Big::operator-(const Big &hi) const
{
    Big res;
    int i, max, rem;
    max = (this->len >  hi.len) ? this->len : hi.len;

    for (i = 0; i < max; i++)
    {
        rem = HUGE_OSN + num[i] - hi.num[i] + res.num[i];
        res.num[i] = rem % HUGE_OSN;
        res.num[i+1] = rem / HUGE_OSN - 1;
    }
    res.len = (res.num[max] == 0) ? max : max+1;

    return res;
}

Big Big::operator*(const Big &hi) const
{
    Big res;
    int i, j, sum;

    for (i = 0; i < this->len; i++)
    {
        for (j = 0; j < hi.len; j++)
        {
            sum = num[i] * hi.num[j] + res.num[i+j];
            res.num[i+j+1] += sum / HUGE_OSN;
            res.num[i+j] = sum % HUGE_OSN;
        }
    }

    res.len = this->len + hi.len;
    while ((res.num[res.len-1] == 0) && (res.len > 1))
        res.len--;

    return res;
}

int Big::operator>=(const Big &hi) const
{
    int max = (this->len >  hi.len) ? this->len : hi.len;
    for (int i = max-1; i >= 0; i--)
        if      (num[i] > hi.num[i]) return 1;
        else if (num[i] < hi.num[i]) return 0;
    return 1;
}

void Big::write(FILE *fp) const
{
    for (int i = len-1; i >= 0; i--)
        fprintf(fp, "%i", num[i]);
    fprintf(fp, "\n");
}

int Big::read(FILE *fp)
{
    zero();
    char str[HUGE_LEN];
    if (fscanf(fp, "%s", str) < 0) return 0;
    len = strlen(str);
    for (int i = len-1; i >= 0; i--)
        num[i] = str[len-1-i] - '0';
    return 1;
}

// main programm

#define IN "in.txt"
#define OUT1 "out1.txt"
#define OUT2 "out2.txt"

#define BYTES 8
#define BITS (BYTES*8)
Big a[BITS];

//    
const Big BASE("7");
//  
#define RAND Big(1 + rand() % 23)

void init()
{
    srand((unsigned int)time(NULL));

    a[0] = BASE * RAND + RAND;
    a[0].write(stdout);
    Big sum = a[0];
    for (int i = 1; i < BITS; i++) {
        a[i] = sum * BASE * RAND + RAND;
        a[i].write(stdout);
        sum = sum + a[i];
    }
}

int main()
{
    printf("[0] start init...\n");
    init();
    printf("[1] done...\n\n");

    FILE *in, *out;
    char x[BYTES];
    Big y;

    printf("[2] encoding...\n");

    in = fopen(IN, "rb");
    out = fopen(OUT1, "wt");

// encoding [start]
//
    while (!feof(in))
    {
        memset(x, 0, sizeof(char)*BYTES);
        if (fread(x, sizeof(char), BYTES, in) < 1) // read
            break;

        y = 0;
        for (int i = 0; i < BITS; i++)
            if ( x[i/8] & (1<<(i%8)) ) // proverka i-bita
                y = y + a[i];

        y.write(out); // write
    }
//
// encoding [end]

    fclose(out);
    fclose(in);

    printf("[3] done.\n\n");
    printf("[4] decoding...\n");

    in = fopen(OUT1, "rt");
    out = fopen(OUT2, "wb");

// decoding [start]
//
    while (!feof(in))
    {
        memset(x, 0, sizeof(char)*BYTES);
        if (!y.read(in)) // read
            break;

        for (int i = BITS-1; i >= 0; i--)
            if (y >= a[i]) {
                x[i/8] |= (1<<(i%8)); // ustanovka i-bita = 1
                y = y - a[i];
            }

       int len = strlen(x);
       fwrite(x, sizeof(char), (len < BYTES ? len : BYTES), out); // write
    }
//
// decoding [end]

    fclose(out);
    fclose(in);

    printf("[5] done.\n\n");

    return 0;
}
